function [alt, lat] = geodet7(rmag, decl)

% solve system of nonlinear equations that define the
% relationship between geocentric and geodetic coordinates

% input

%  rmag = geocentric radius (kilometers)
%  decl = geocentric declination (radians)

% output

%  altitude = geodetic altitude (kilometers)
%  latitude = geodetic latitude (radians)

% Orbital Mechanics with MATLAB

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global req rmag_wrk decl_wrk

% load "working" values for geocentric radius and declination

rmag_wrk = rmag;

decl_wrk = decl;

% initial guess for geodetic altitude (kilometers)

xg(1) = rmag - req;

% initial guess for geodetic latitude (radians)

xg(2) = decl;

% define algorithm options

options = optimoptions('fsolve', 'algorithm', 'trust-region-dogleg');

% solve the system of nonlinear equations using fsolve

xsol = fsolve(@geodet7_snle, xg, options);

% geodetic solution

alt = xsol(1);

lat = xsol(2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = geodet7_snle(x)

% system of nonlinear equations defining the relationship
% between geocentric and geodetic coordinates

% input

%  x = current argument vector

% output

%  f = nonlinear system evaluated at x

% Orbital Mechanics with MATLAB

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global req flat rmag_wrk decl_wrk

alt = x(1);

lat = x(2);

% calculate "geodetic" constants

c = req / sqrt (1.0 - (2.0 * flat - flat ^ 2) * sin(lat) ^ 2);

s = c * (1.0 - flat) ^ 2;

% system of nonlinear equations

f(1) = (c + alt) * cos(lat) - rmag_wrk * cos(decl_wrk);

f(2) = (s + alt) * sin(lat) - rmag_wrk * sin(decl_wrk);

